
-- ͻ
--        D E S I G N   E N G I N E R I N G              DS        
--             S O F T W A R E                           ͼ        
--                                                                    
--         Package     Os2.Rexx                                       
--                                                                    
--         Author :  Leonid Dulman     1995,1997                      
--                                                                    
--              GNU ADA RUNTIME LIBRARY (GNARL) COMPONENTS            
--                                                                    
--                  REXX  support                                     
--                                                                    
-- ͼ

with Interfaces.C;         use Interfaces.C;
with Interfaces.C.Strings; use Interfaces.C.Strings;

package Os2.Rexx is
pragma Preelaborate (Rexx);

--  ͻ
--                                                                     
--     #define:              To include:                               
--                                                                     
--     INCL_REXXSAA          Complete Rexx support                     
--     INCL_RXSUBCOM         Rexx subcommand handler support           
--     INCL_RXSHV            Rexx shared variable pool support         
--     INCL_RXFUNC           Rexx external function support            
--     INCL_RXSYSEXIT        Rexx system exit support                  
--     INCL_RXMACRO          Rexx macro space support                  
--     INCL_RXARI            Rexx asynchronous Trace/Halt support      
--                                                                     
--  ͼ

    --                            Common
    -- This section defines return codes and constants that are the
    -- same for both 16-bit and 32-bit REXX calls.

    --    Structure for external interface string (RXSTRING)

type  RXSTRING is record          -- rxstr
  strlength         :ULONG  ;           --   length of string
  strptr            :PCH    ;           --   pointer to string
end record; --  RXSTRING;

type PRXSTRING     is access all RXSTRING;       -- pointer to a RXSTRING

 RXAUTOBUFLEN :constant Ulong :=  256;

    --    Structure for system exit block (RXSYSEXIT) 32-bit

type  RXSYSEXIT is record      -- syse
sysexit_name      :PSZ   ;     -- subcom enviro for sysexit
sysexit_code      :LONG  ;     -- sysexit function code
end record; --  RXSYSEXIT;

type PRXSYSEXIT is access all RXSYSEXIT;         -- pointer to a RXSYSEXIT

    --   Macros for RXSTRING manipulation

--  #define RXNULLSTRING(r)      (!(r).strptr)
--  #define RXZEROLENSTRING(r)   ((r).strptr && !(r).strlength)
--  #define RXVALIDSTRING(r)     ((r).strptr && (r).strlength)
--  #define RXSTRLEN(r)          (RXNULLSTRING(r)?0L:(r).strlength)
--  #define RXSTRPTR(r)          (r).strptr
--  #define MAKERXSTRING(r,p,l)  is record(r).strptr=(PCH)p;(r).strlength=(ULONG)l;}
    --   Call type codes for use on interpreter startup

 RXCOMMAND    : constant Ulong :=   0 ;  -- Program called as Command
 RXSUBROUTINE : constant Ulong :=   1 ;  -- Program called as Subroutin
 RXFUNCTION   : constant Ulong :=   2 ;  -- Program called as Function

    --      Subcommand Interface defines
    --      Drop Authority for RXSUBCOM interface

 RXSUBCOM_DROPPABLE : constant Ulong :=   16#00#;   -- handler to be dropped by all
 RXSUBCOM_NONDROP   : constant Ulong :=   16#01#;   -- process with same PID as the
--                                        -- registrant may drop environ

--  Return Codes from RXSUBCOM interface

 RXSUBCOM_ISREG      : constant Ulong := 01;   -- Subcommand is registered
 RXSUBCOM_ERROR      : constant Ulong := 01;    -- Subcommand Ended in Error
 RXSUBCOM_FAILURE    : constant Ulong := 02;    -- Subcommand Ended in Failure
 RXSUBCOM_BADENTRY   : constant Ulong := 1001;    -- Invalid Entry Conditions
 RXSUBCOM_NOEMEM     : constant Ulong := 1002;    -- Insuff stor to complete req
 RXSUBCOM_BADTYPE    : constant Ulong := 1003;    -- Bad registration type.
 RXSUBCOM_NOTINIT    : constant Ulong := 1004;    -- API system not initialized.
 RXSUBCOM_OK         : constant Ulong :=  0 ;     -- Function Complete
 RXSUBCOM_DUP        : constant Ulong := 10 ;     -- Duplicate Environment Name-
                               -- but Registration Completed
 RXSUBCOM_MAXREG     : constant Ulong := 20;      -- Cannot register more
                               -- handlers
 RXSUBCOM_NOTREG     : constant Ulong := 30;      -- Name Not Registered
 RXSUBCOM_NOCANDROP  : constant Ulong := 40;      -- Name not droppable
 RXSUBCOM_LOADERR    : constant Ulong := 50;      -- Could not load function
 RXSUBCOM_NOPROC     : constant Ulong :=127;      -- RXSUBCOM routine - not found

--     Shared Variable Pool Interface defines
--     Function Codes for Variable Pool Interface (shvcode)

 RXSHV_SET   : constant Ulong := 00; -- Set var from given value
 RXSHV_FETCH : constant Ulong := 01; -- Copy value of var to buffer
 RXSHV_DROPV : constant Ulong := 02; -- Drop variable
 RXSHV_SYSET : constant Ulong := 03; -- Symbolic name Set variable
 RXSHV_SYFET : constant Ulong := 04; -- Symbolic name Fetch variable
 RXSHV_SYDRO : constant Ulong := 05; -- Symbolic name Drop variable
 RXSHV_NEXTV : constant Ulong := 06; -- Fetch "next" variable
 RXSHV_PRIV  : constant Ulong := 07; -- Fetch private information
 RXSHV_EXIT  : constant Ulong := 08; -- Set function exit value

--   Return Codes for Variable Pool Interface

 RXSHV_NOAVL : constant Ulong := 144; -- Interface not available

--  Return Code Flags for Variable Pool Interface (shvret)

 RXSHV_OK    : constant Ulong := 16#00#;  -- Execution was OK
 RXSHV_NEWV  : constant Ulong := 16#01#;  -- Variable did not exist
 RXSHV_LVAR  : constant Ulong := 16#02#;  -- Last var trans via SHVNEXTV
 RXSHV_TRUN  : constant Ulong := 16#04#;  -- Truncation occurred-Fetch
 RXSHV_BADN  : constant Ulong := 16#08#;  -- Invalid variable name
 RXSHV_MEMFL : constant Ulong := 16#10#;  -- Out of memory failure
 RXSHV_BADF  : constant Ulong := 16#80#;  -- Invalid funct code (shvcode)

    --  Structure of Shared Variable Request Block (SHVBLOCK)
type SHVBLOCK ;
type PSHVBLOCK is access SHVBLOCK;

type  SHVBLOCK is record            -- shvb
 pshvnext        :PSHVBLOCK ;       -- pointer to the next block
 shvname         :RXSTRING  ;       -- Pointer to the name buffer
 shvvalue        :RXSTRING  ;       -- Pointer to the value buffer
 shvnamelen      :ULONG     ;       -- Length of the name value
 shvvaluelen     :ULONG     ;       -- Length of the fetch value
 shvcode         :UCHAR     ;       -- Function code for this block
 shvret          :UCHAR     ;       -- Individual Return Code Flags
end record; --   SHVBLOCK;

    --    External Function Interface
    --    Registration Type Identifiers for Available Function Table

 RXFUNC_DYNALINK  : constant Ulong := 1;  -- Function Available in DLL
 RXFUNC_CALLENTRY : constant Ulong := 2;   -- Registered as mem entry pt.

    --   Return Codes from RxFunction interface

 RXFUNC_OK          : constant Ulong :=   0 ;      -- REXX-API Call Successful
 RXFUNC_DEFINED     : constant Ulong :=  10 ;      -- Function Defined in AFT
 RXFUNC_NOMEM       : constant Ulong :=  20 ;      -- Not Enough Mem to Add
 RXFUNC_NOTREG      : constant Ulong :=  30 ;      -- Funct Not Registered in AFT
 RXFUNC_MODNOTFND   : constant Ulong :=  40 ;      -- Funct Dll Module Not Found
 RXFUNC_ENTNOTFND   : constant Ulong :=  50 ;      -- Funct Entry Point Not Found
 RXFUNC_NOTINIT     : constant Ulong :=  60 ;      -- API not initialized
 RXFUNC_BADTYPE     : constant Ulong :=  70 ;      -- Bad function type

    --   System Exits defines
    --    Drop Authority for Rexx Exit interface

 RXEXIT_DROPPABLE  : constant long :=   00 ;   -- handler to be dropped by all
 RXEXIT_NONDROP    : constant long :=   01 ;   -- process with same PID as the
                                          -- registrant may drop environ
    --**    Exit return actions

 RXEXIT_HANDLED      : constant long := 0  ;     -- Exit handled exit event
 RXEXIT_NOT_HANDLED  : constant long := 1  ;     -- Exit passes on exit event
 RXEXIT_RAISE_ERROR  : constant long :=  -1 ;    -- Exit handler error occurred

--    Return Codes from RXEXIT interface

 RXEXIT_ISREG      : constant long :=     01 ;   -- Exit is registered
 RXEXIT_ERROR      : constant long :=     01 ;   -- Exit Ended in Error
 RXEXIT_FAILURE    : constant long :=     02 ;   -- Exit Ended in Failure
 RXEXIT_BADENTRY   : constant long :=   1001 ;   -- Invalid Entry Conditions
 RXEXIT_NOEMEM     : constant long :=   1002 ;   -- Insuff stor to complete req
 RXEXIT_BADTYPE    : constant long :=   1003 ;   -- Bad registration type.
 RXEXIT_NOTINIT    : constant long :=   1004 ;   -- API system not initialized.
 RXEXIT_OK         : constant long :=    0  ;    -- Function Complete
 RXEXIT_DUP        : constant long :=   10  ;    -- Duplicate Exit Name-
                                                 -- but Registration Completed
 RXEXIT_MAXREG     : constant long :=   20  ;    -- Cannot register more
                                                 -- handlers
 RXEXIT_NOTREG     : constant long :=   30  ;    -- Name Not Registered
 RXEXIT_NOCANDROP  : constant long :=   40  ;    -- Name not droppable
 RXEXIT_LOADERR    : constant long :=   50  ;    -- Could not load function
 RXEXIT_NOPROC     : constant long :=  127  ;    -- RXEXIT routine - not found

--  System Exit function and sub-function definitions

 RXENDLST    : constant long := 0  ;  -- End of exit list.
 RXFNC       : constant long := 2  ;  -- Process external functions.
 RXFNCCAL    : constant long := 1  ;  -- subcode value.
 RXCMD       : constant long := 3  ;  -- Process host commands.
 RXCMDHST    : constant long := 1  ;  -- subcode value.
 RXMSQ       : constant long := 4  ;  -- Manipulate queue.
 RXMSQPLL    : constant long := 1  ;  -- Pull a line from queue
 RXMSQPSH    : constant long := 2  ;  -- Place a line on queue
 RXMSQSIZ    : constant long := 3  ;  -- Return num of lines on queue
 RXMSQNAM    : constant long := 20 ;  -- Set active queue name
 RXSIO       : constant long := 5  ;  -- Session I/O.
 RXSIOSAY    : constant long := 1  ;  -- SAY a line to STDOUT
 RXSIOTRC    : constant long := 2  ;  -- Trace output
 RXSIOTRD    : constant long := 3  ;  -- Read from char stream
 RXSIODTR    : constant long := 4  ;  -- DEBUG read from char stream
 RXSIOTLL    : constant long := 5  ;  -- Return linelength(N/A OS/2)
 RXHLT       : constant long := 7  ;  -- Halt processing.
 RXHLTCLR    : constant long := 1  ;  -- Clear HALT indicator
 RXHLTTST    : constant long := 2  ;  -- Test HALT indicator
 RXTRC       : constant long := 8  ;  -- Test ext trace indicator.
 RXTRCTST    : constant long := 1  ;  -- subcode value.
 RXINI       : constant long := 9  ;  -- Initialization processing.
 RXINIEXT    : constant long := 1  ;  -- subcode value.
 RXTER       : constant long := 10 ;  -- Termination processing.
 RXTEREXT    : constant long := 1  ;  -- subcode value.
 RXNOOFEXITS : constant long := 11 ;  -- 1 + largest exit number.

subtype  PEXIT is PUCHAR;                 -- ptr to exit parameter block

    --    Asynchronous Request Interface defines
    --    Return Codes from Asynchronous Request interface

 RXARI_OK                 : constant long :=  0; -- Interface completed
 RXARI_NOT_FOUND          : constant long :=  1; -- Target program not found
 RXARI_PROCESSING_ERROR   : constant long :=  2; -- Error processing request

--  Macro Space Interface defines
--  Registration Search Order Flags

 RXMACRO_SEARCH_BEFORE   : constant long := 1; -- Beginning of search order
 RXMACRO_SEARCH_AFTER    : constant long := 2; -- End of search order

--    Return Codes from RxMacroSpace interface

 RXMACRO_OK                 : constant long :=0; -- Macro interface completed
 RXMACRO_NO_STORAGE         : constant long :=1; -- Not Enough Storage Available
 RXMACRO_NOT_FOUND          : constant long :=2; -- Requested function not found
 RXMACRO_EXTENSION_REQUIRED : constant long :=3; -- File ext required for save
 RXMACRO_ALREADY_EXISTS     : constant long :=4; -- Macro functions exist
 RXMACRO_FILE_ERROR         : constant long :=5; -- File I/O error in save/load
 RXMACRO_SIGNATURE_ERROR    : constant long :=6; -- Incorrect format for load
 RXMACRO_SOURCE_NOT_FOUND   : constant long :=7; -- Requested cannot be found
 RXMACRO_INVALID_POSITION   : constant long :=8; -- Invalid search order pos
 RXMACRO_NOT_INIT           : constant long :=9; -- API not initialized

    --                            32-bit
    --    Main Entry Point to the REXXSAA Interpreter

 function RexxStart (
   name     :LONG ;                        -- Num of args passed to rexx
  pargs     :PRXSTRING;                    -- Array of args passed to rex
  ppath     :PSZ;                          -- [d:][path] filename[.ext]
  ploc      :PRXSTRING;                    -- Loc of rexx proc in memory
  penv      :PSZ;                          -- ASCIIZ initial environment.
  tip       :LONG ;                        -- type (command;subrtn;funct)
  pexit     :PRXSYSEXIT;                   -- SysExit env. names &  codes
  pret      :PUSHORT;                      -- Ret code from if numeric
  pretn     :PRXSTRING ) return LONG;      -- Retvalue from the rexx proc
pragma Import(c,RexxStart, Link_name=>"_RexxStart");

    --    Subcommand Interface
    -- This typedef simplifies coding of a Subcommand handler.
    --  #pragma linkage(RexxSubcomHandler;system)
type RexxSubcomHandler is access function (pstr:PRXSTRING;
                                           push:PUSHORT;
                                           prs :PRXSTRING) return ULONG;

    --   RexxRegisterSubcomDll  Register a DLL entry point
    --   as a Subcommand handler

function RexxRegisterSubcomDll (
 pname      :PSZ;                          -- Name of subcom handler
 pndll      :PSZ;                          -- Name of DLL
 pprocndll  :PSZ;                          -- Name of procedure in DLL
 parea      :PUCHAR;                       -- User area
 pdrop      :ULONG  ) return APIRET;       -- Drop authority.
pragma Import(c,RexxRegisterSubcomDll, Link_name=>"_RexxRegisterSubcomDll");

    --   RexxRegisterSubcomExe -- Register an EXE entry point
    --   as a Subcommand handler

function RexxRegisterSubcomExe (
      pname :PSZ;                          -- Name of subcom handler
      pexe  :PSZ;                          -- address of handler in EXE
      parea :PUCHAR) return APIRET;        -- User area
pragma Import(c,RexxRegisterSubcomExe, Link_name=>"_RexxRegisterSubcomExe");

    --    RexxQuerySubcom - Query an environment for Existance

function RexxQuerySubcom(
penv        :PSZ;                          -- Name of the Environment
pdll        :PSZ;                          -- DLL Module Name
pstor       :PUSHORT;                      -- Stor for existance code
parea       :PUCHAR ) return APIRET;       -- Stor for user word
pragma Import(c,RexxQuerySubcom, Link_name=>"_RexxQuerySubcom");

    --    RexxDeregisterSubcom - Drop registration of a Subcommand environment

function RexxDeregisterSubcom(
penv        :PSZ;                          -- Name of the Environment
pdll        :PSZ ) return APIRET;          -- DLL Module Name
pragma Import(c,RexxDeregisterSubcom, Link_name=>"_RexxDeregisterSubcom");

    --    Shared Variable Pool Interface
    --    RexxVariablePool - Request Variable Pool Service

function RexxVariablePool(plist:PSHVBLOCK) return APIRET; -- Pointer to list of SHVBLOCKs
pragma Import(c,RexxVariablePool, Link_name=>"_RexxVariablePool");

    -- This typedef simplifies coding of an External Function.
type RexxFunctionHandler is access function(p1:PUCHAR;
                                     p2:ULONG;
                                     p3: PRXSTRING;
                                     p4:PSZ;
                                     p5: PRXSTRING) return ULONG;

    --    RexxRegisterFunctionDll - Register a function in the AFT

function RexxRegisterFunctionDll (
  pname    :PSZ;                           -- Name of function to add
  pdllfn   :PSZ;                           -- Dll file name (if in dll)
  pdll     :PSZ) return APIRET;            -- Entry in dll
pragma Import(c,RexxRegisterFunctionDll, Link_name=>"_RexxRegisterFunctionDll");

    --    RexxRegisterFunctionExe - Register a function in the AFT

function RexxRegisterFunctionExe (
    pnadd  :PSZ;                           -- Name of function to add
    pentr  :PSZ) return APIRET;            -- Entry point in EXE
pragma Import(c,RexxRegisterFunctionExe, Link_name=>"_RexxRegisterFunctionExe");

    --   RexxDeregisterFunction - Delete a function from the AFT

function RexxDeregisterFunction (pname:PSZ ) return APIRET; -- Name of function to remove
pragma Import(c,RexxDeregisterFunction, Link_name=>"_RexxDeregisterFunction");

    --    RexxQueryFunction - Scan the AFT for a function

function RexxQueryFunction (pname :PSZ ) return APIRET; -- Name of function to find
pragma Import(c,RexxQueryFunction, Link_name=>"_RexxQueryFunction");

    --   System Exits
    --   Subfunction RXFNCCAL - External Function Calls

-- type   RXFNC_FLAGS is record          -- fl
--       unsigned rxfferr  : 1;              -- Invalid call to routine.
--       unsigned rxffnfnd : 1;              -- Function not found.
--       unsigned rxffsub  : 1;              -- Called as a subroutine
--end record; --  RXFNC_FLAGS ;
subtype RXFNC_FLAGS is BYTE;

type   RXFNCCAL_PARM is record        -- fnc
   rxfnc_fl      : RXFNC_FLAGS       ;     -- function flags
   rxfnc_name    : PUCHAR            ;       -- Pointer to function name.
   rxfnc_namel   : USHORT            ;      -- Length of function name.
   rxfnc_que     : PUCHAR            ;        -- Current queue name.
   rxfnc_quel    : USHORT            ;       -- Length of queue name.
   rxfnc_argc    : USHORT            ;       -- Number of args in list.
   rxfnc_argv    : PRXSTRING         ;       -- Pointer to argument list.
   rxfnc_retc    : RXSTRING          ;       -- Return value.
end record; --  RXFNCCAL_PARM;
type PRXFNCCAL_PARM is access all  RXFNCCAL_PARM ;

    --    Subfunction RXCMDHST -- Process Host Commands

--type Packed  RXCMD_FLAGS is record         -- fl
--       unsigned rxfcfail : 1;              -- Command failed.
--       unsigned rxfcerr  : 1;              -- Command ERROR occurred.
--end record; --  RXCMD_FLAGS;
subtype RXCMD_FLAGS is BYTE;

type  RXCMDHST_PARM is record        -- rx
  rxcmd_fl           :RXCMD_FLAGS      ;      -- error/failure flags
  rxcmd_address      :PUCHAR           ;    -- Pointer to address name.
  rxcmd_addressl     :USHORT           ;   -- Length of address name.
  rxcmd_dll          :PUCHAR           ;        -- dll name for command.
  rxcmd_dll_len      :USHORT           ;    -- Length of dll name.
  rxcmd_command      :RXSTRING         ;    -- The command string.
  rxcmd_retc         :RXSTRING         ;       -- Pointer to return buffer
end record; --  RXCMDHST_PARM;
type PRXCMDHST_PARM is access all RXCMDHST_PARM;

    --     Subfunction RXMSQPLL -- Pull Entry from Queue

type  RXMSQPLL_PARM is record        -- pll
  rxmsq_retc    :RXSTRING          ;       -- Pointer to dequeued entry
                                           -- buffer.  User allocated.
end record; -- RXMSQPLL_PARM;


    --    Subfunction RXMSQPSH -- Push Entry on Queue

-- type Packed  RXMSQ_FLAGS is record          -- fl
--        unsigned rxfmlifo : 1;              -- Stack entry LIFO if set
-- end record; --  RXMSQ_FLAGS;
subtype RXMSQ_FLAGS  is byte;

type  RXMSQPSH_PARM is record        -- psh
   rxmsq_fl      :RXMSQ_FLAGS       ;      -- LIFO/FIFO flag
   rxmsq_value   :RXSTRING          ;      -- The entry to be pushed.
end record; --  RXMSQPSH_PARM;
type PRXMSQPSH_PARM is access all  RXMSQPSH_PARM;

    --    Subfunction RXMSQSIZ -- Return the Current Queue Size

type  RXMSQSIZ_PARM is record        -- siz
   rxmsq_size   :ULONG             ;       -- Number of Lines in Queue
end record; --  RXMSQSIZ_PARM;
type PRXMSQSIZ_PARM is access all RXMSQSIZ_PARM;

    --    Subfunction RXMSQNAM -- Set Current Queue Name

type  RXMSQNAM_PARM is record        -- nam
   rxmsq_name :  RXSTRING          ;       -- RXSTRING containing
                                           -- queue name.
end record; --  RXMSQNAM_PARM;
type PRXMSQNAM_PARM is access all RXMSQNAM_PARM;

    --    Subfunction RXSIOSAY -- Perform SAY Clause

type  RXSIOSAY_PARM is record        -- say
  rxsio_string    :RXSTRING          ;     -- String to display.
end record; --  RXSIOSAY_PARM;
type PRXSIOSAY_PARM is access all RXSIOSAY_PARM;

    --    Subfunction RXSIOTRC -- Write Trace Output

type  RXSIOTRC_PARM is record -- trcparm
  rxsio_string   : RXSTRING          ;     -- Trace line to display.
end record; --  RXSIOTRC_PARM;
type PRXSIOTRC_PARM is access all  RXSIOTRC_PARM;

    --    Subfunction RXSIOTRD    Read Input from the Terminal

type  RXSIOTRD_PARM is record        -- trd
  rxsiotrd_retc  :  RXSTRING          ;    -- RXSTRING for output.
end record; --  RXSIOTRD_PARM;
type PRXSIOTRD_PARM is access all RXSIOTRD_PARM;

    --    Subfunction RXSIODTR -- Read Debug Input from the Terminal

type  RXSIODTR_PARM is record        -- dtr
  rxsiodtr_retc   : RXSTRING          ;    -- RXSTRING for output.
end record; --  RXSIODTR_PARM;
type PRXSIODTR_PARM is access all  RXSIODTR_PARM;

    --    Subfunction RXHSTTST -- Test for HALT Condition

-- type  RXHLT_FLAGS is record          -- fl Halt flag
--       unsigned rxfhhalt : 1;              -- Set if HALT occurred.
--end record; --  RXHLT_FLAGS;
subtype RXHLT_FLAGS is  BYTE;

type  RXHLTTST_PARM is record        -- tst
  rxhlt_fl      : RXHLT_FLAGS ;            -- Set if HALT occurred
end record; --  RXHLTTST_PARM;
type PRXHLTTST_PARM is access all    RXHLTTST_PARM;

    --    Subfunction RXTRCTST -- Test for TRACE Condition

-- type  RXTRC_FLAGS is record          -- fl Trace flags
--        unsigned rxftrace : 1;              -- Set to run external trace.
-- end record; --  RXTRC_FLAGS;
subtype RXTRC_FLAGS is BYTE;

type  RXTRCTST_PARM is record        -- tst
  rxtrc_fl      : RXTRC_FLAGS ;            -- Set to run external trace
end record; --  RXTRCTST_PARM;
type PRXTRCTST_PARM is access all RXTRCTST_PARM;

    -- This typedef simplifies coding of an Exit handler.
type RexxExitHandler is access function (p1:LONG;
                                         p2:LONG;
                                         p3:PEXIT) return APIRET;

    --      RexxRegisterExitDll - Register a system exit.

function RexxRegisterExitDll (
          nh:PSZ;                          -- Name of the exit handler
        ndll:PSZ;                          -- Name of the DLL
         np :PSZ;                          -- Name of the procedure
       area :PUCHAR;                       -- User area
       drop :ULONG ) return APIRET;        -- Drop authority
pragma Import(c,RexxRegisterExitDll, Link_name=>"_RexxRegisterExitDll");

    --      RexxRegisterExitExe - Register a system exit.

function RexxRegisterExitExe (
          nh:PSZ;                          -- Name of the exit handler
         pnh:PSZ;                          -- Address of exit handler
       parea:PUCHAR) return APIRET;        -- User area
pragma Import(c,RexxRegisterExitExe, Link_name=>"_RexxRegisterExitExe");

    --    RexxDeregisterExit - Drop registration of a system exit.

function RexxDeregisterExit (
  exitn     :PSZ;                          -- Exit name
  ndll      :PSZ ) return APIRET;          -- DLL module name
pragma Import(c,RexxDeregisterExit, Link_name=>"_RexxDeregisterExit");

    --    RexxQueryExit - Query an exit for existance.

function RexxQueryExit (
          p1:PSZ;                          -- Exit name
          p2:PSZ;                          -- DLL Module name.
          p3:PUSHORT;                      -- Existance flag.
          p4:PUCHAR ) return APIRET;       -- User data.
pragma Import(c,RexxQueryExit, Link_name=>"_RexxQueryExit");

    --    Asynchronous Request Interface
    --    RexxSetHalt - Request Program Halt

function RexxSetHalt(
         p1: PID;                         -- Process Id
         p2: TID) return APIRET;          -- Thread Id
pragma Import(c,RexxSetHalt, Link_name=>"_RexxSetHalt");

    --    RexxSetTrace - Request Program Trace

function RexxSetTrace(
          p1:PID;                         -- Process Id
          p2:TID) return APIRET;          -- Thread Id
pragma Import(c,RexxSetTrace, Link_name=>"_RexxSetTrace");

    --    RexxResetTrace - Turn Off Program Trace

function RexxResetTrace(
         p1: PID;                         -- Process Id
         p2: TID) return APIRET;          -- Thread Id
pragma Import(c,RexxResetTrace, Link_name=>"_RexxResetTrace");

    --    RexxAddMacro - Register a function in the Macro Space

function RexxAddMacro (
         p1 :PSZ;                         -- Function to add or change
         p2 :PSZ;                         -- Name of file to get function
         p3 :ULONG  ) return APIRET;       -- Flag indicating search pos
pragma Import(c,RexxAddMacro, Link_name=>"_RexxAddMacro");

    --    RexxDropMacro - Remove a function from the Macro Space

function RexxDropMacro (
          p :PSZ ) return APIRET;     -- Name of function to remove
pragma Import(c,RexxDropMacro, Link_name=>"_RexxDropMacro");

    --    RexxSaveMacroSpace - Save Macro Space functions to a file

function RexxSaveMacroSpace (
          p1:ULONG ;                      -- Argument count (0==save all)
          p2:PSZ  ;                       -- List of funct names to save
          p3:PSZ) return APIRET;          -- File to save functions in
pragma Import(c,RexxSaveMacroSpace, Link_name=>"_RexxSaveMacroSpace");

    --    RexxLoadMacroSpace - Load Macro Space functions from a file

function RexxLoadMacroSpace (
         p1 :ULONG ;                      -- Argument count (0==load all)
         p2 :PSZ  ;                       -- List of funct names to load
         p3 :PSZ) return APIRET;          -- File to load functions from
pragma Import(c,RexxLoadMacroSpace, Link_name=>"_RexxLoadMacroSpace");

    --    RexxQueryMacro - Find a function's search-order position

function RexxQueryMacro (
          p1:PSZ;                         -- Function to search for
          p2:PUSHORT ) return APIRET;     -- Ptr for position flag return
pragma Import(c,RexxQueryMacro, Link_name=>"_RexxQueryMacro");

    --    RexxReorderMacro - Change a function's search-order
    --                            position

function RexxReorderMacro(
         p1 :PSZ;                         -- Name of funct change order
         p2 :ULONG  ) return APIRET;      -- New position for function
pragma Import(c,RexxReorderMacro, Link_name=>"_RexxReorderMacro");

    --    RexxClearMacroSpace - Remove all functions from a MacroSpace

function RexxClearMacroSpace return APIRET;  -- No Arguments.
pragma Import(c,RexxClearMacroSpace, Link_name=>"_RexxClearMacroSpace");
end Os2.Rexx;
